@@ -22,6 +22,8 @@ from .fixtures import django_db_use_migrations  # noqa
 from .fixtures import django_db_keepdb  # noqa
 from .fixtures import django_db_createdb  # noqa
 from .fixtures import django_db_modify_db_settings  # noqa
+from .fixtures import django_db_modify_db_settings_parallel_suffix  # noqa
+from .fixtures import django_db_modify_db_settings_tox_suffix  # noqa
 from .fixtures import django_db_modify_db_settings_xdist_suffix  # noqa
 from .fixtures import _live_server_helper  # noqa
 from .fixtures import admin_client  # noqa
@@ -245,14 +247,14 @@ def pytest_load_initial_conftests(early_config, parser, args):
     early_config.addinivalue_line(
         "markers",
         "django_db(transaction=False): Mark the test as using "
-        "the django test database.  The *transaction* argument marks will "
+        "the Django test database.  The *transaction* argument marks will "
         "allow you to use real transactions in the test like Django's "
         "TransactionTestCase.",
     )
     early_config.addinivalue_line(
         "markers",
         "urls(modstr): Use a different URLconf for this test, similar to "
-        "the `urls` attribute of Django `TestCase` objects.  *modstr* is "
+        "the `urls` attribute of Django's `TestCase` objects.  *modstr* is "
         "a string specifying the module of a URL config, e.g. "
         '"my_app.test_urls".',
     )
@@ -425,6 +427,30 @@ def pytest_runtest_setup(item):
             _disable_class_methods(item.cls)
 
 
+def pytest_collection_modifyitems(session, config, items):
+    def get_order_number(test):
+        marker_db = test.get_closest_marker('django_db')
+        if marker_db:
+            transaction = validate_django_db(marker_db)[0]
+            if transaction is True:
+                return 1
+        else:
+            transaction = None
+
+        fixtures = getattr(test, 'fixturenames', [])
+        if "transactional_db" in fixtures:
+            return 1
+
+        if transaction is False:
+            return 0
+        if "db" in fixtures:
+            return 0
+
+        return 2
+
+    items[:] = sorted(items, key=get_order_number)
+
+
 @pytest.fixture(autouse=True, scope="session")
 def django_test_environment(request):
     """
@@ -488,43 +514,44 @@ def _django_db_marker(request):
 @pytest.fixture(autouse=True, scope="class")
 def _django_setup_unittest(request, django_db_blocker):
     """Setup a django unittest, internal to pytest-django."""
-    if django_settings_is_configured() and is_django_unittest(request):
-        request.getfixturevalue("django_test_environment")
-        request.getfixturevalue("django_db_setup")
+    if not django_settings_is_configured() or not is_django_unittest(request):
+        yield
+        return
 
-        django_db_blocker.unblock()
+    from _pytest.unittest import TestCaseFunction
 
-        cls = request.node.cls
+    if "debug" in TestCaseFunction.runtest.__code__.co_names:
+        # Fix pytest (https://github.com/pytest-dev/pytest/issues/5991), only
+        # if "self._testcase.debug()" is being used (forward compatible).
+        from _pytest.monkeypatch import MonkeyPatch
 
-        # implement missing (as of 1.10) debug() method for django's TestCase
-        # see pytest-dev/pytest-django#406
-        def _cleaning_debug(self):
-            testMethod = getattr(self, self._testMethodName)
-            skipped = getattr(self.__class__, "__unittest_skip__", False) or getattr(
-                testMethod, "__unittest_skip__", False
-            )
+        def non_debugging_runtest(self):
+            self._testcase(result=self)
 
-            if not skipped:
-                self._pre_setup()
-            super(cls, self).debug()
-            if not skipped:
-                self._post_teardown()
+        mp_debug = MonkeyPatch()
+        mp_debug.setattr("_pytest.unittest.TestCaseFunction.runtest", non_debugging_runtest)
+    else:
+        mp_debug = None
 
-        cls.debug = _cleaning_debug
+    request.getfixturevalue("django_db_setup")
 
+    cls = request.node.cls
+
+    with django_db_blocker.unblock():
         if _handle_unittest_methods:
             _restore_class_methods(cls)
             cls.setUpClass()
             _disable_class_methods(cls)
 
-            def teardown():
-                _restore_class_methods(cls)
-                cls.tearDownClass()
-                django_db_blocker.restore()
+            yield
 
-            request.addfinalizer(teardown)
+            _restore_class_methods(cls)
+            cls.tearDownClass()
         else:
-            request.addfinalizer(django_db_blocker.restore)
+            yield
+
+    if mp_debug:
+        mp_debug.undo()
 
 
 @pytest.fixture(scope="function", autouse=True)
@@ -745,7 +772,7 @@ class _DatabaseBlocker(object):
     def _blocking_wrapper(*args, **kwargs):
         __tracebackhide__ = True
         __tracebackhide__  # Silence pyflakes
-        pytest.fail(
+        raise RuntimeError(
             "Database access not allowed, "
             'use the "django_db" mark, or the '
             '"db" or "transactional_db" fixtures to enable it.'
